#!/usr/bin/env ruby

# Copyright (c) 2007, Gregory Fleischer (gfleischer@gmail.com)
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#   2. Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in
#      the documentation and/or other materials provided with the
#      distribution.
#   3. The names of the authors may not be used to endorse or promote
#      products derived from this software without specific prior
#      written permission.
#  
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


# simple TFTP client
require 'libtftp.rb'

####
if __FILE__ == $0
  host, addr, port = nil, nil, 69
  blksize = 512
  tsize = FALSE
  timeout = 5
  mode = "octet"
  trace = FALSE
  client = TFTPClient.new

  if ARGV.length > 0
    h = ARGV.shift
    begin
      addr = IPSocket.getaddress(h)
      host = h
    rescue SocketError
      # ignore bad hostname
      puts "bad host: %s" % h
      exit 1
    end
  end
  if ARGV.length > 0
    p = ARGV.shift
    port = p.to_i
    if 0 == port 
      puts "bad port: %s" % p
      exit 1
    end
  end


  # read from prompt
  begin
    if STDIN.isatty
      print '> '
    end
    while cmd = gets
      cmd.strip!
      next if cmd.empty?
      if "?" == cmd or "help" == cmd
        puts %{commands:
connect  -    connect to host
ascii    -    ascii mode
binary   -    binary mode
mode     -    netascii | octet | mail
get      -    receive
put      -    send
blksize  -    set blocksize
tsize    -    toggle/set tsize
timeout  -    set timeout 
trace    -    enable tracing
status   -    display status
quit     -    quit
help     -    help
?        -    help

}
      elsif "quit" == cmd
        exit 0
      elsif "status" == cmd
        puts %{host: #{if host.nil? then "no host" else "%s:%s (%s)" % [host,port,addr]end}
mode: #{mode}
blksize: #{blksize}
timeout: #{timeout}
tsize: #{tsize ? "yes" : "no"}
trace: #{trace ? "yes" : "no"}
}
      elsif m = cmd.match(/blksize\s+(\d+)$/)
        begin
          blksize = m[1].to_i
        rescue Exception => e
          puts "error: {e}"
        end
        puts "using blksize #{blksize}"
      elsif m = cmd.match(/blksize\s*$/)
        puts "using blksize #{blksize}"
      elsif "tsize" == cmd
        tsize = tsize ? FALSE : TRUE
        puts "tsize is #{tsize ? 'on' : 'off'}"
      elsif m = cmd.match(/tsize\s+(on|off)$/)
        if "on" == m[1]
          tsize = TRUE
        else
          tsize = FALSE
        end
        puts "tsize is #{tsize ? 'on' : 'off'}"
      elsif "trace" == cmd
        trace = trace ? FALSE : TRUE
        puts "trace is #{trace ? 'on' : 'off'}"
      elsif m = cmd.match(/trace\s+(on|off)$/)
        if "on" == m[1]
          trace = TRUE
        else
          trace = FALSE
        end
        puts "trace is #{trace ? 'on' : 'off'}"
      elsif m = cmd.match(/timeout\s+(\d+)$/) 
        begin
          timeout = m[1].to_i
        rescue Exception => e
          puts "error: " + e
        end
        puts "using timeout #{timeout}"
      elsif m = cmd.match(/timeout\s*$/)
        puts "using timeout #{timeout}"
      elsif m = cmd.match(/conn(?:ect)?\s+([-A-Za-z0-9._]+)(?:\s+(\d+))?$/)
        h = m[1]
        begin
          addr = IPSocket.getaddress(h)
          host = h
          if not m[2].nil?
            port = m[2].to_i
          else
            port = 69
          end
        rescue SocketError
          # ignore bad hostname
          puts "bad host: %s" % h
        end
      elsif m = cmd.match(/conn(?:ect)?\s*$/)
        puts "use: connect <hostname> [port]"
      elsif m = cmd.match(/mode\s+(netascii|octet|mail)$/)
        mode = m[1]
      elsif m = cmd.match(/mode\s*$/)
        puts "using mode #{mode}"
      elsif cmd.match(/asc(?:ii)?$/)
        mode = "netascii"
        puts "using mode #{mode}"
      elsif cmd.match(/bin(?:ary)?$/)
        mode = "octet"
        puts "using mode #{mode}"
      elsif m = cmd.match(/get\s+(\S+?)(?:\s+(\S+?))?$/)
        remotefile = m[1]
        if m[2].nil?
          localfile = File.basename(remotefile)
        else
          localfile = m[2]
        end
        if host.nil?
          puts "no host specified"
        else
          resp = client.read(host, addr, port, mode, remotefile, localfile, blksize, timeout, tsize, trace)
          if not resp.nil? and resp then
            puts "#{localfile} received"
          else
            puts "receive failed"
          end
        end
      elsif m = cmd.match(/put\s+(\S+?)(?:\s+(\S+?))?$/)
        localfile = m[1]
        remotefile = m[2] || localfile
        if host.nil?
          puts "no host specified"
        elsif not File.exists?(localfile) 
          puts "file %s does not exist" % localfile
        else
          resp = client.write(host, addr, port, mode, localfile, remotefile, blksize, timeout, tsize, trace)
          if not resp.nil? and resp then
            puts "#{localfile} transmitted"
          else
            puts "transmit failed"
          end
        end
      else
        puts "unknown command: %s" % cmd
      end
      
      if STDIN.isatty
        print '> '
      end

    end

  rescue Interrupt
  end

  if STDIN.isatty
    puts "\ngoodbye"
  end
end

# eof
