#!/usr/bin/env ruby

# Copyright (c) 2007, Gregory Fleischer (gfleischer@gmail.com)
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#   2. Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in
#      the documentation and/or other materials provided with the
#      distribution.
#   3. The names of the authors may not be used to endorse or promote
#      products derived from this software without specific prior
#      written permission.
#  
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

require 'libtftp.rb'
require 'getoptlong'

#########
if __FILE__ == $0

  def print_usage
    puts ""
    puts "usage: #{$0} [ OPTION ]..."
    puts ""
    puts "--help,     -h              display this help"
    puts "--addr,     -a <addr>       listen address [ default: any ]"
    puts "--port,     -p <port>       listen port [ default: 6969 ]"
    puts "--dir,      -d <dir>        server directory [ default: /srv/tftp ]"
    puts "--write,    -w <yes|no>     write allowed [ default: no ]"
    puts "--restrict, -r <yes|no>     restrict files to local directory [ default: yes ]"
    puts "--verbose,  -v              verbose tracing"
    puts ""
  end


  opts = GetoptLong.new(
                        [ '--help', '-h', GetoptLong::NO_ARGUMENT ],
                        [ '--port', '-p', GetoptLong::REQUIRED_ARGUMENT ],
                        [ '--addr', '-a', GetoptLong::REQUIRED_ARGUMENT ],
                        [ '--dir', '-d', GetoptLong::REQUIRED_ARGUMENT ],
                        [ '--write', '-w', GetoptLong::REQUIRED_ARGUMENT ],
                        [ '--restrict', '-r', GetoptLong::REQUIRED_ARGUMENT ],
                        [ '--verbose', '-v', GetoptLong::NO_ARGUMENT ]
                        )
  addr = "0.0.0.0"
  port = 6969
  dir = "/srv/tftp"
  verbose = FALSE
  write = FALSE
  restrict = TRUE
  opts.each do |opt, arg|
    case opt
    when "--help"
      print_usage
      exit(0)
    when "--port"
      port = arg.to_i
    when "--addr"
      addr = arg
    when "--dir"
      dir = arg
    when "--verbose"
      verbose = TRUE
    when "--write"
      if "yes" == arg
        write = TRUE
      elsif "no" == arg
        write = FALSE
      else
        print_usage
        exit(1)
      end
    when "--restrict"
      if "yes" == arg
        restrict = TRUE
      elsif "no" == arg
        restrict = FALSE
      else
        print_usage
        exit(1)
      end
    end
  end

  puts "starting server on #{addr}:#{port} in #{dir}; write: #{write}, restrict: #{restrict}"
  begin
    Dir.chroot(dir)
    Dir.chdir("/")
  rescue Errno::EPERM
    # probably not running as root
    Dir.chdir(dir)
  end

  # define an appropriate callback
  request_callback = Proc.new do |socket, client, opcode, filename, mode, options|
    if restrict and 
        # validate absolute or relative paths
        (
         '/'[0] == filename[0] or filename.match(/\.\.[\/\\]/)
         ) 
    then
      TFTPError.AccessViolation(socket, client)
      puts "denying restricted request for #{filename}"
      next FALSE
    end
    if TFTPOpCode::WRQ == opcode and not write then
      TFTPError.AccessViolation(socket, client)
      puts "denying write request for #{filename}"
      next FALSE
    end
    TRUE
  end

  server = TFTPServer.new(addr, port, verbose, request_callback)
  server.listen

end

# eof
